@@ -1,5 +1,5 @@
 ;;; gnus-xmas.el --- Gnus functions for XEmacs
-;; Copyright (C) 1995,96 Free Software Foundation, Inc.
+;; Copyright (C) 1995,96,97 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
 ;; Keywords: news
@@ -26,7 +26,6 @@
 ;;; Code:
 
 (require 'text-props)
-(eval-when-compile (require 'cl))
 (defvar menu-bar-mode (featurep 'menubar))
 (require 'messagexmas)
 
@@ -36,8 +35,8 @@ If this variable is nil, Gnus will try to locate the directory
 automatically.")
 
 (defvar gnus-xmas-logo-color-alist
-  '((flame "#cc3300" "#ff2200") 
-    (pine "#c0cc93" "#f8ffb8") 
+  '((flame "#cc3300" "#ff2200")
+    (pine "#c0cc93" "#f8ffb8")
     (moss "#a1cc93" "#d2ffb8")
     (irish "#04cc90" "#05ff97")
     (sky "#049acc" "#05deff")
@@ -50,7 +49,7 @@ automatically.")
     (september "#bf9900" "#ffcc00"))
   "Color alist used for the Gnus logo.")
 
-(defvar gnus-xmas-logo-color-style 'september
+(defvar gnus-xmas-logo-color-style 'flame
   "Color styles used for the Gnus logo.")
 
 (defvar gnus-xmas-logo-colors
@@ -118,7 +117,7 @@ asynchronously.	 The compressed face will be piped to this command.")
 (defun gnus-xmas-set-text-properties (start end props &optional buffer)
   "You should NEVER use this function.  It is ideologically blasphemous.
 It is provided only to ease porting of broken FSF Emacs programs."
-  (if (stringp buffer) 
+  (if (stringp buffer)
       nil
     (map-extents (lambda (extent ignored)
 		   (remove-text-properties 
@@ -131,19 +130,26 @@ It is provided only to ease porting of broken FSF Emacs programs."
 (defun gnus-xmas-highlight-selected-summary ()
   ;; Highlight selected article in summary buffer
   (when gnus-summary-selected-face
-    (if gnus-newsgroup-selected-overlay
-	(delete-extent gnus-newsgroup-selected-overlay))
+    (when gnus-newsgroup-selected-overlay
+      (delete-extent gnus-newsgroup-selected-overlay))
     (setq gnus-newsgroup-selected-overlay 
 	  (make-extent (gnus-point-at-bol) (gnus-point-at-eol)))
     (set-extent-face gnus-newsgroup-selected-overlay
 		     gnus-summary-selected-face)))
 
+(defvar gnus-xmas-force-redisplay t
+  "If non-nil, force a redisplay before recentering the summary buffer.
+This is ugly, but it works around a bug in `window-displayed-height'.")
+
 (defun gnus-xmas-summary-recenter ()
   "\"Center\" point in the summary window.
 If `gnus-auto-center-summary' is nil, or the article buffer isn't
 displayed, no centering will be performed."
   ;; Suggested by earle@mahendo.JPL.NASA.GOV (Greg Earle).
   ;; Recenter only when requested.  Suggested by popovich@park.cs.columbia.edu.
+  ;; Force redisplay to get properly computed window height.
+  (when gnus-xmas-force-redisplay
+    (sit-for 0))
   (when gnus-auto-center-summary
     (let* ((height (if (fboundp 'window-displayed-height)
 		       (window-displayed-height)
@@ -161,8 +167,7 @@ displayed, no centering will be performed."
 	;; possible valid number, or the second line from the top,
 	;; whichever is the least.
 	(set-window-start
-	 window (min bottom (save-excursion 
-			      (forward-line (- top)) (point)))))
+	 window (min bottom (save-excursion (forward-line (- top)) (point)))))
       ;; Do horizontal recentering while we're at it.
       (when (and (get-buffer-window (current-buffer) t)
 		 (not (eq gnus-auto-center-summary 'vertical)))
@@ -197,7 +202,8 @@ call it with the value of the `gnus-data' text property."
   (let* ((pos (event-closest-point event))
 	 (data (get-text-property pos 'gnus-data))
 	 (fun (get-text-property pos 'gnus-callback)))
-    (if fun (funcall fun data))))
+    (when fun
+      (funcall fun data))))
 
 (defun gnus-xmas-move-overlay (extent start end &optional buffer)
   (set-extent-endpoints extent start end))
@@ -205,9 +211,9 @@ call it with the value of the `gnus-data' text property."
 ;; Fixed by Christopher Davis <ckd@loiosh.kei.com>.
 (defun gnus-xmas-article-add-button (from to fun &optional data)
   "Create a button between FROM and TO with callback FUN and data DATA."
-  (and gnus-article-button-face
-       (gnus-overlay-put (gnus-make-overlay from to) 
-			 'face gnus-article-button-face))
+  (when gnus-article-button-face
+    (gnus-overlay-put (gnus-make-overlay from to)
+		      'face gnus-article-button-face))
   (gnus-add-text-properties 
    from to
    (nconc
@@ -249,21 +255,18 @@ call it with the value of the `gnus-data' text property."
              (next-bottom-edge (car (cdr (cdr (cdr 
                                                (window-pixel-edges 
 						this-window)))))))
-        (if (< bottom-edge next-bottom-edge)
-            (progn
-              (setq bottom-edge next-bottom-edge)
-              (setq lowest-window this-window)))
+        (when (< bottom-edge next-bottom-edge)
+	  (setq bottom-edge next-bottom-edge)
+	  (setq lowest-window this-window))
 
         (select-window this-window)
-        (if (eq last-window this-window)
-            (progn
-              (select-window lowest-window)
-              (setq window-search nil)))))))
+        (when (eq last-window this-window)
+	  (select-window lowest-window)
+	  (setq window-search nil))))))
 
 (defmacro gnus-xmas-menu-add (type &rest menus)
   `(gnus-xmas-menu-add-1 ',type ',menus))
 (put 'gnus-xmas-menu-add 'lisp-indent-function 1)
-(put 'gnus-xmas-menu-add 'lisp-indent-hook 1)
 
 (defun gnus-xmas-menu-add-1 (type menus)
   (when (and menu-bar-mode
@@ -293,6 +296,10 @@ call it with the value of the `gnus-data' text property."
   (gnus-xmas-menu-add pick
     gnus-pick-menu))
 
+(defun gnus-xmas-topic-menu-add ()
+  (gnus-xmas-menu-add topic
+    gnus-topic-menu))
+
 (defun gnus-xmas-binary-menu-add ()
   (gnus-xmas-menu-add binary
     gnus-binary-menu))
@@ -315,12 +322,12 @@ call it with the value of the `gnus-data' text property."
 
 (defun gnus-xmas-read-event-char ()
   "Get the next event."
-  (let ((event (next-event)))
+  (let ((event (next-command-event)))
+    (sit-for 0)
     ;; We junk all non-key events.  Is this naughty?
     (while (not (key-press-event-p event))
-      (setq event (next-event)))
+      (setq event (next-command-event)))
     (cons (and (key-press-event-p event) 
-	      ; (numberp (event-key event))
 	       (event-to-character event)) 
 	  event)))
 
@@ -365,15 +372,16 @@ call it with the value of the `gnus-data' text property."
 (defun gnus-xmas-define ()
   (setq gnus-mouse-2 [button2])
 
-  (or (memq 'underline (face-list))
-      (and (fboundp 'make-face)
-	   (funcall (intern "make-face") 'underline)))
+  (unless (memq 'underline (face-list))
+    (and (fboundp 'make-face)
+	 (funcall (intern "make-face") 'underline)))
   ;; Must avoid calling set-face-underline-p directly, because it
   ;; is a defsubst in emacs19, and will make the .elc files non
   ;; portable!
-  (or (face-differs-from-default-p 'underline)
-      (funcall (intern "set-face-underline-p") 'underline t))
+  (unless (face-differs-from-default-p 'underline)
+    (funcall (intern "set-face-underline-p") 'underline t))
 
+  (fset 'gnus-characterp 'characterp)
   (fset 'gnus-make-overlay 'make-extent)
   (fset 'gnus-overlay-put 'set-extent-property)
   (fset 'gnus-move-overlay 'gnus-xmas-move-overlay)
@@ -383,10 +391,14 @@ call it with the value of the `gnus-data' text property."
   (fset 'gnus-put-text-property 'gnus-xmas-put-text-property)
       
   (require 'text-props)
-  (if (< emacs-minor-version 14)
-      (fset 'gnus-set-text-properties 'gnus-xmas-set-text-properties))
+  (when (< emacs-minor-version 14)
+    (fset 'gnus-set-text-properties 'gnus-xmas-set-text-properties))
 
-  (or (boundp 'standard-display-table) (setq standard-display-table nil))
+  (when (fboundp 'turn-off-scroll-in-place)
+    (add-hook 'gnus-article-mode-hook 'turn-off-scroll-in-place))
+
+  (unless (boundp 'standard-display-table)
+    (setq standard-display-table nil))
 
   (defvar gnus-mouse-face-prop 'highlight)
 
@@ -406,57 +418,13 @@ call it with the value of the `gnus-data' text property."
       (if (compiled-function-p fval)
 	  (list 'funcall fval)
 	(cons 'progn (cdr (cdr fval))))))
-      
-  ;; Fix by "jeff (j.d.) sparkes" <jsparkes@bnr.ca>.
-  (defvar gnus-display-type (device-class)
-    "A symbol indicating the display Emacs is running under.
-The symbol should be one of `color', `grayscale' or `mono'. If Emacs
-guesses this display attribute wrongly, either set this variable in
-your `~/.emacs' or set the resource `Emacs.displayType' in your
-`~/.Xdefaults'. See also `gnus-background-mode'.
-
-This is a meta-variable that will affect what default values other
-variables get.  You would normally not change this variable, but
-pounce directly on the real variables themselves.")
-
 
   (fset 'gnus-x-color-values 
 	(if (fboundp 'x-color-values)
 	    'x-color-values
 	  (lambda (color)
 	    (color-instance-rgb-components
-	     (make-color-instance color)))))
-    
-  (defvar gnus-background-mode 
-    (let* ((bg-resource 
-	    (condition-case ()
-		(x-get-resource ".backgroundMode" "BackgroundMode" 'string)
-	      (error nil)))
-	   (params (frame-parameters))
-	   (color (condition-case ()
-		      (or (assq 'background-color params)
-			  (color-instance-name
-			   (specifier-instance
-			    (face-background 'default))))
-		    (error nil))))
-      (cond (bg-resource (intern (downcase bg-resource)))
-	    ((and color
-		  (< (apply '+ (gnus-x-color-values color))
-		     (/ (apply '+ (gnus-x-color-values "white")) 3)))
-	     'dark)
-	    (t 'light)))
-    "A symbol indicating the Emacs background brightness.
-The symbol should be one of `light' or `dark'.
-If Emacs guesses this frame attribute wrongly, either set this variable in
-your `~/.emacs' or set the resource `Emacs.backgroundMode' in your
-`~/.Xdefaults'.
-See also `gnus-display-type'.
-
-This is a meta-variable that will affect what default values other
-variables get.  You would normally not change this variable, but
-pounce directly on the real variables themselves.")
-  )
-
+	     (make-color-instance color))))))
 
 
 (defun gnus-xmas-redefine ()
@@ -477,9 +445,9 @@ pounce directly on the real variables themselves.")
   (fset 'gnus-make-local-hook 'make-local-variable)
   (fset 'gnus-add-hook 'gnus-xmas-add-hook)
   (fset 'gnus-character-to-event 'character-to-event)
-  (fset 'gnus-article-show-hidden-text 'gnus-xmas-article-show-hidden-text)
   (fset 'gnus-mode-line-buffer-identification
 	'gnus-xmas-mode-line-buffer-identification)
+  (fset 'gnus-key-press-event-p 'key-press-event-p)
 
   (add-hook 'gnus-group-mode-hook 'gnus-xmas-group-menu-add)
   (add-hook 'gnus-summary-mode-hook 'gnus-xmas-summary-menu-add)
@@ -487,6 +455,7 @@ pounce directly on the real variables themselves.")
   (add-hook 'gnus-score-mode-hook 'gnus-xmas-score-menu-add)
 
   (add-hook 'gnus-pick-mode-hook 'gnus-xmas-pick-menu-add)
+  (add-hook 'gnus-topic-mode-hook 'gnus-xmas-topic-menu-add)
   (add-hook 'gnus-tree-mode-hook 'gnus-xmas-tree-menu-add)
   (add-hook 'gnus-binary-mode-hook 'gnus-xmas-binary-menu-add)
   (add-hook 'gnus-grouplens-mode-hook 'gnus-xmas-grouplens-menu-add)
@@ -498,7 +467,8 @@ pounce directly on the real variables themselves.")
 
   (when (and (<= emacs-major-version 19)
 	     (<= emacs-minor-version 13))
-    (setq gnus-article-x-face-too-ugly (if (eq (device-type) 'tty) "."))
+    (setq gnus-article-x-face-too-ugly (when (eq (device-type) 'tty)
+					 "."))
     (fset 'gnus-highlight-selected-summary
 	  'gnus-xmas-highlight-selected-summary)
     (fset 'gnus-group-remove-excess-properties
@@ -507,8 +477,7 @@ pounce directly on the real variables themselves.")
 	  'gnus-xmas-topic-remove-excess-properties)
     (fset 'gnus-mode-line-buffer-identification 'identity)
     (unless (boundp 'shell-command-switch)
-      (setq shell-command-switch "-c"))
-    ))
+      (setq shell-command-switch "-c"))))
 
 
 ;;; XEmacs logo and toolbar.
@@ -570,7 +539,7 @@ pounce directly on the real variables themselves.")
 " 
 	       ""))
       ;; And then hack it.
-      (gnus-indent-rigidly (point-min) (point-max) 
+      (gnus-indent-rigidly (point-min) (point-max)
 			   (/ (max (- (window-width) (or x 46)) 0) 2))
       (goto-char (point-min))
       (forward-line 1)
@@ -580,13 +549,11 @@ pounce directly on the real variables themselves.")
 	(insert (make-string (max 0 (* 2 (/ rest 3))) ?\n))))
     ;; Fontify some.
     (goto-char (point-min))
-    (and (search-forward "Praxis" nil t)
-	 (put-text-property (match-beginning 0) (match-end 0) 'face 'bold))
+    (put-text-property (point-min) (point-max) 'face 'gnus-splash-face)
     (goto-char (point-min))
-    (let* ((mode-string (gnus-group-set-mode-line)))
-      (setq modeline-buffer-identification 
-	    (list (concat gnus-version ": *Group*")))
-      (set-buffer-modified-p t))))
+    (setq modeline-buffer-identification 
+	  (list (concat gnus-version ": *Group*")))
+    (set-buffer-modified-p t)))
 
 
 ;;; The toolbar.
@@ -600,22 +567,22 @@ If it is non-nil, it must be a toolbar.  The five legal values are
 `right-toolbar', and `left-toolbar'.")
 
 (defvar gnus-group-toolbar 
-  '(
-    [gnus-group-get-new-news gnus-group-get-new-news t "Get new news"]
+  '([gnus-group-get-new-news gnus-group-get-new-news t "Get new news"]
     [gnus-group-get-new-news-this-group 
      gnus-group-get-new-news-this-group t "Get new news in this group"]
     [gnus-group-catchup-current 
      gnus-group-catchup-current t "Catchup group"]
     [gnus-group-describe-group 
      gnus-group-describe-group t "Describe group"]
+    [gnus-group-unsubscribe gnus-group-unsubscribe t "Unsubscribe group"]
+    [gnus-group-subscribe gnus-group-subscribe t "Subscribe group"]
     [gnus-group-kill-group gnus-group-kill-group t "Kill group"]
     [gnus-group-exit gnus-group-exit t "Exit Gnus"]
     )
   "The group buffer toolbar.")
 
 (defvar gnus-summary-toolbar 
-  '(
-    [gnus-summary-prev-unread 
+  '([gnus-summary-prev-unread 
      gnus-summary-prev-unread-article t "Prev unread article"]
     [gnus-summary-next-unread 
      gnus-summary-next-unread-article t "Next unread article"]
@@ -642,6 +609,8 @@ If it is non-nil, it must be a toolbar.  The five legal values are
      gnus-uu-post-news t "Post an uuencoded article"]
     [gnus-summary-cancel-article
      gnus-summary-cancel-article t "Cancel article"]
+    [gnus-summary-catchup
+     gnus-summary-catchup t "Catchup"]
     [gnus-summary-catchup-and-exit
      gnus-summary-catchup-and-exit t "Catchup and exit"]
     [gnus-summary-exit gnus-summary-exit t "Exit this summary"]
@@ -655,7 +624,7 @@ If it is non-nil, it must be a toolbar.  The five legal values are
     [gnus-summary-next-unread 
      gnus-summary-next-unread-article t "Next unread article"]
     [gnus-summary-mail-reply gnus-summary-reply t "Reply"]
-    [gnus-summary-mail-get gnus-mail-get t "Message get"]
+;    [gnus-summary-mail-get gnus-mail-get t "Message get"]
     [gnus-summary-mail-originate gnus-summary-post-news t "Originate"]
     [gnus-summary-mail-save gnus-summary-save-article t "Save"]
     [gnus-summary-mail-copy gnus-summary-copy-article t "Copy message"]
@@ -671,6 +640,8 @@ If it is non-nil, it must be a toolbar.  The five legal values are
      gnus-summary-save-article-file t "Save article in file"]
     [gnus-summary-save-article
      gnus-summary-save-article t "Save article"]
+    [gnus-summary-catchup
+     gnus-summary-catchup t "Catchup"]
     [gnus-summary-catchup-and-exit
      gnus-summary-catchup-and-exit t "Catchup and exit"]
     [gnus-summary-exit gnus-summary-exit t "Exit this summary"]
@@ -735,36 +706,50 @@ XEmacs compatibility workaround."
       (set-extent-begin-glyph 
        (make-extent (point) (1+ (point))) xface-glyph))))
 
-(defun gnus-xmas-article-show-hidden-text (type &optional hide)
-  "Show all hidden text of type TYPE.
-If HIDE, hide the text instead."
-  (save-excursion
-    (set-buffer gnus-article-buffer)
-    (let ((buffer-read-only nil)
-	  (inhibit-point-motion-hooks t)
-	  (beg (point-min)))
-      (while (gnus-goto-char (text-property-any
-			      beg (point-max) 'gnus-type type))
-	(setq beg (point))
-	(forward-char)
-	(if hide
-	    (gnus-hide-text beg (point) gnus-hidden-properties)
-	  (gnus-unhide-text beg (point)))
-	(setq beg (point)))
-      (save-window-excursion
-	(select-window (get-buffer-window (current-buffer)))
-	(recenter))
-      t)))
+(defvar gnus-xmas-pointer-glyph 
+  (progn
+    (setq gnus-xmas-glyph-directory (message-xmas-find-glyph-directory "gnus"))
+    (make-pointer-glyph (concat gnus-xmas-glyph-directory "gnus-pointer."
+				(if (featurep 'xpm) "xpm" "xbm")))))
+
+(defvar gnus-xmas-modeline-left-extent 
+  (let ((ext (copy-extent modeline-buffer-id-left-extent)))
+    ;(set-extent-property ext 'pointer gnus-xmas-pointer-glyph)
+    ext))
+      
+(defvar gnus-xmas-modeline-right-extent 
+  (let ((ext (copy-extent modeline-buffer-id-right-extent)))
+    ;(set-extent-property ext 'pointer gnus-xmas-pointer-glyph)
+    ext))
+
+(defvar gnus-xmas-modeline-glyph
+  (progn
+    (setq gnus-xmas-glyph-directory (message-xmas-find-glyph-directory "gnus"))
+    (let* ((file (concat gnus-xmas-glyph-directory "gnus-pointer."
+			 (if (featurep 'xpm) "xpm" "xbm")))
+	   (glyph (make-glyph file)))
+      (when (and (featurep 'x)
+		 (file-exists-p file))
+	(set-glyph-face glyph 'modeline-buffer-id))
+      (set-glyph-property glyph 'image (cons 'tty "Gnus:"))
+      glyph)))
 
 (defun gnus-xmas-mode-line-buffer-identification (line)
   (let ((line (car line))
 	chop)
     (if (not (stringp line))
 	(list line)
-      (unless (setq chop (string-match ":" line))
-	(setq chop (/ (length line) 2)))
-      (list (cons modeline-buffer-id-left-extent (substring line 0 chop))
-	    (cons modeline-buffer-id-right-extent (substring line chop))))))
+      (when (string-match "^Gnus:" line)
+	(setq chop (match-end 0))
+	(list 
+	 (if gnus-xmas-modeline-glyph
+	     (cons gnus-xmas-modeline-left-extent gnus-xmas-modeline-glyph)
+	   (cons gnus-xmas-modeline-left-extent (substring line 0 chop)))
+	 (cons gnus-xmas-modeline-right-extent (substring line chop)))))))
+
+(defun gnus-xmas-splash ()
+  (when (eq (device-type) 'x)
+    (gnus-splash)))
 
 (provide 'gnus-xmas)
 
